import pytest
from sib.project import Project

def test_load_project_for_development():

    # load project from previous testing phase
    project = Project('sibproject', '/tmp/sib-dev', [])

    assert project.load('test', 'test', 'test@test.io')



import pytest
from sib.package import Package

def test_create_package_for_development():

    # load project from previous testing phase
    package = Package('pkg', '/tmp/sib-dev/pkg')
    assert package.create()



import click
import os
from urllib.parse import urlparse
from . import __version__
from .project import Project
from .package import Package

# click entrypoint
@click.group()
def main():
    """Startin'Blox installer"""
    pass

@main.command()
@click.argument('name', nargs=1)
@click.argument('directory', required=False)
@click.option('-m', '--modules', multiple=True, help='List of modules to install')
@click.option('--site-url', default='http://localhost:8000', help='Setup the site URL')
@click.option('--production', is_flag=True, default=False, help='Configure project for production')
@click.option('--db-host', help='Configure host for the database')
@click.option('--db-name', help='Configure name for the database')
@click.option('--db-user', help='Configure user for the database')
@click.option('--db-pass', help='Configure password for the database')
@click.option('--smtp-host', help='Configure host for the SMTP server')
@click.option('--smtp-user', help='Configure user for the SMTP server')
@click.option('--smtp-pass', help='Configure password for the SMTP server')
@click.option('--smtp-port', default=587, help='Configure port for the SMTP server')
@click.option('--smtp-tls', default=True, help='Configure TLS for the SMTP server')
def startproject(name, production, site_url, directory, modules, db_host, db_name, db_user, db_pass, smtp_host, smtp_user, smtp_pass, smtp_port, smtp_tls):

    """Start a new startin'blox project"""

    # set absolute path to project directory
    if directory:
        directory = os.path.abspath(directory)
    else:
        # set a directory from project name in pwd
        directory = os.path.abspath(name)

    # split modules and distributions in dependencies
    deps = []
    for module in modules:
        try:
            pkg, dist = module.split('@', 1)
        except ValueError:
            pkg = module
            dist = module

        deps.append((pkg, dist))

    # designed mandatory option for producrion
    if production:
        if not db_host:
            print('Error: Missing option "--db-host" for production')
            return
        if not db_name:
            print('Error: Missing option "--db-name" for production')
            return
        if not db_user:
            print('Error: Missing option "--db-user" for production')
            return
        if not db_pass:
            print('Error: Missing option "--db-pass" for production')
            return

    # build allowed_hosts from site_url
    allowed_hosts = urlparse(site_url).hostname

    project = Project(
        name=name,
        folder=directory,
        modules=deps
    )

    project.create(
        production=production,
        site_url=site_url,
        allowed_hosts=allowed_hosts,
        db_host=db_host,
        db_name=db_name,
        db_user=db_user,
        db_pass=db_pass,
        smtp_host=smtp_host,
        smtp_user=smtp_user,
        smtp_pass=smtp_pass,
        smtp_port=smtp_port,
        smtp_tls=smtp_tls
    )

@main.command()
@click.argument('directory', required=False)
@click.option('--production', is_flag=True, default=False, help='Configure project for production')
@click.option('--admin-name', help='Setup admin username')
@click.option('--admin-pass', help='Setup admin password')
@click.option('--admin-email', help='Setup admin email')
def initproject(directory, production, admin_name, admin_pass, admin_email):

    """Initialize a startin'blox project"""

    # set absolute path to project directory
    if directory:
        directory = os.path.abspath(directory)
    else:
        # get path from current dir
        directory = os.getcwd()

    # resolve project name from directory
    name = os.path.basename(directory)

    # designed mandatory option for producrion
    if production:
        if not admin_name:
            print('Error: Missing option "--admin-name" for production')
        if not admin_email:
            print('Error: Missing option "--admin-email" for production')
        if not admin_pass:
            print('Error: Missing option "--admin-pass" for production')
    else:
        if not admin_name:
            admin_name = 'admin'
        if not admin_email:
            admin_email = ''
        if not admin_pass:
            admin_pass = 'admin'

    project = Project(name, directory)

    project.load(admin_name, admin_pass, admin_email)


@main.command()
@click.argument('name', nargs=1)
@click.argument('directory', required=False)
def startpackage(name, directory):

    """Create a new startin'blox package"""

    # set absolute path to package directory (given directory is the project path)
    if directory:
        directory = os.path.join(os.path.abspath(directory), name)
    else:
        # get path from current dir
        directory = os.path.join(os.getcwd(), name)

    package = Package(name, directory)
    package.create()

@main.command()
def version():
    """Print module version"""
    click.echo(__version__)



import subprocess
import sys
from os import mkdir, path, environ
from importlib import import_module
from pkg_resources import resource_filename


class Project(object):

    """
    This class initialize a new SIB project by installing the required components and configuring the project
    """

    def __init__(self, name, folder, modules=[]):

        """
        Initialization
        modules is a list of tuple (package, distrib)
        """

        # set project params
        self.project_name = name
        self.project_folder = folder

        # store packages (python module or packages)
        self.packages = [m[0] for m in modules]

        # store distribution (pip distributions)
        self.dists = [m[1] for m in modules]
        self.dists.append('djangoldp')  # add required distribution

    def get_template(self, production):

        """Return the path of django project template from package resouces"""

        if production:
            return resource_filename(__name__, 'templates/production')

        return resource_filename(__name__, 'templates/development')

    def create(self, site_url, allowed_hosts, production=False, db_host=None, db_name=None, db_user=None, db_pass=None, smtp_host=None, smtp_user=None, smtp_pass=None, smtp_port=587, smtp_tls=True):

        """Create the SIB project"""

        try:
            # create project folder
            mkdir(self.project_folder)

            # add psycopg2 for production
            if production:
                self.dists.append('psycopg2_binary')

            # install distributions
            cmd = ['pip', 'install']
            cmd.extend(self.dists)
            subprocess.run(cmd).check_returncode()

        except subprocess.CalledProcessError as e:
            print('Installation failed: {}'.format(e))
            return False

        try:
            from django.core import management
            from django.template.exceptions import TemplateSyntaxError

            # init django project
            management.call_command(
                'startproject',
                self.project_name,
                self.project_folder,
                template=self.get_template(production),
                packages=self.packages,
                site_url=site_url,
                allowed_hosts=allowed_hosts,
                db_host=db_host,
                db_name=db_name,
                db_user=db_user,
                db_pass=db_pass,
                smtp_host=smtp_host,
                smtp_user=smtp_user,
                smtp_pass=smtp_pass
            )

        except ModuleNotFoundError as e:
            print('[ERROR] Django not imported: {}'.format(e))
            return False

        except TemplateSyntaxError as e:
            print('[ERROR] Project template is wrong: {}'.format(e))
            return False

        return True

    def load(self, admin_name='admin', admin_pass='admin', admin_email=''):

        try:
            # load sib project settings
            sys.path.append(self.project_folder)
            sib_settings = import_module(self.project_name + '.settings')
            environ.setdefault('DJANGO_SETTINGS_MODULE', sib_settings.__name__)

            # django imports
            import django
            from django.core.exceptions import ImproperlyConfigured
            from django.core.management.base import CommandError
            from django.core import management

            # setup the django sib project
            django.setup()

        except ModuleNotFoundError as e:
            print('[ERROR] {}'.format(e))
            return False

        except ImportError as e:
            print('[ERROR] Project not initialized: {}'.format(e))
            return False

        try:
            # migrate data
            management.call_command('migrate', interactive=False)

        except ImproperlyConfigured as e:
            print('[ERROR] Django configuration failed: {}'.format(e))
            return False

        except CommandError as e:
            print('[ERROR] Django migration failed: {}'.format(e))
            return False

        try:
            # create superuser
            from django.contrib.auth.models import User
            User.objects.create_superuser(admin_name, admin_email, admin_pass)
        except ValueError as e:
            print('[ERROR] Super user creation failed: {}'.format(e))
            return False

        try:
            # creatersakey
            management.call_command('creatersakey', interactive=False)

        except CommandError:
            pass

        return True



import pytest
from os import path
from sib.project import Project

@pytest.fixture
def init_with_fake_modules(tmp_path):
    modules = [('pkg1', 'dist1'), ('pkg2', 'dist2')]
    folder = path.join(tmp_path,'sibtest')
    return Project('sibproject', folder, modules)

@pytest.fixture
def init_without_modules(tmp_path):
    modules = []
    folder = path.join(tmp_path,'sibtest')
    return Project('sibproject', folder, modules)

def test_init_project_with_fake_modules(init_with_fake_modules):
    assert init_with_fake_modules.project_name == 'sibproject'
    assert init_with_fake_modules.packages == ['pkg1', 'pkg2']
    assert init_with_fake_modules.dists == ['dist1', 'dist2', 'djangoldp']

def test_init_project_without_modules(init_without_modules):
    assert init_without_modules.project_name == 'sibproject'
    assert init_without_modules.packages == []
    assert init_without_modules.dists == ['djangoldp']

def test_create_project_for_development():

    # make project persistent for load_project testing
    project = Project('sibproject', '/tmp/sib-dev', [])

    project.create(
        site_url='http://localhost:8000',
        allowed_hosts='localhost',
        production=False
    )

    # test djangoldp import
    try:
        import djangoldp
        assert True
    except ImportError:
        assert False

    # FIXME: test template

def test_create_project_with_private_module(tmp_path):

    modules = [('myapp', 'git+https://git.happy-dev.fr/startinblox/devops/siblab-python')]
    folder = path.join(tmp_path, 'sibtest')
    project = Project('sibproject', folder, modules)

    project.create(
        site_url='http://localhost:8000',
        allowed_hosts='localhost',
        production=False
    )

    # test to import private test repository
    try:
        import myapp
        assert True
    except ImportError:
        assert False
